from ..utils.file import read_file


s = "here is test \n that goes into a file \n"
f = open('temp.txt', 'w')
f.write(s)
f.close()


def run_test():

    out = read_file('temp.txt')
    if out != ['here is test \n', ' that goes into a file \n']:
        raise ValueError('read_file() test failed')



from numpy import asarray, zeros
from keras.preprocessing.text import Tokenizer
from keras.preprocessing.sequence import pad_sequences


def keras_tokenize(texts, num_words=None, maxlen=12, padding='post'):

    '''TOKENIZE

    Create a word index and returns padded docs in return.

    '''

    tokenizer = Tokenizer(num_words=num_words)
    tokenizer.fit_on_texts(texts)
    encoded_docs = tokenizer.texts_to_sequences(texts)
    word_index = tokenizer.word_index

    x = pad_sequences(encoded_docs, maxlen=maxlen, padding=padding)
    vocab_size = max(list(word_index.values())) + 1

    return x, vocab_size, word_index


def keras_embeddings(trained_vectors):

    '''KERAS EMBEDDING INDEX

    Takes in trained vectors (e.g. GloVe) and returns
    a Keras embeddings index.

    PARAMS
    ------

    trained_vectors :: a file with the trained vectors

    '''

    embeddings_index = {}

    with open(trained_vectors, encoding='utf-8') as f:

        for line in f:

            values = line.split()
            word = values[0]
            try:
                coefs = asarray(values[1:], dtype='float32')
            except ValueError:
                pass
            embeddings_index[word] = coefs

    vector_dims = len(list(embeddings_index.values())[0])

    return embeddings_index, vector_dims


def keras_embedding_matrix(word_index,
                           vector_dims,
                           embeddings_index,
                           max_num_words=None):

    if max_num_words is None:
        max_num_words = len(word_index) + 1

    num_words = min(max_num_words, len(word_index) + 1)
    embedding_matrix = zeros((num_words, vector_dims))

    for word, i in word_index.items():
        if i >= num_words:
            continue
        embedding_vector = embeddings_index.get(word)

        if embedding_vector is not None:
            embedding_matrix[i] = embedding_vector

    return embedding_matrix



import spacy as sp
try:
    import en_core_web_sm
except ModuleNotFoundError:
    pass

from gensim.models import KeyedVectors, FastText
from gensim.scripts import glove2word2vec


class Signs:

    '''LOAD SPACY MODEL

    The default option 'en' loads the spacy default vectors.
    Alternative a local vector file can be loaded or one of the
    other spacy vectors.

    trained_vectors :: name or path to the pre-trained vector file
    backend :: either 'spacy' or 'gensim'
    mode :: applies to backend 'gensim'. The options are 'glove', 'word2vec',
            and 'fasttext'
    temp :: filename for glove to word2vec conversion
    binary :: if the vector file format is binary or not

    '''

    def __init__(self,
                 trained_vectors='en',
                 backend='spacy',
                 mode=None,
                 temp='signs_temp.sig',
                 binary=False):

        self.backend = backend
        self.mode = mode
        self.temp = temp
        self.binary = binary
        self.trained_vectors = trained_vectors
        self._null = self.load_model()

    def load_model(self):

        if self.backend == 'spacy':
            self.model = sp.load(self.trained_vectors)

        elif self.backend == 'gensim':

            if self.mode is None:
                self.mode = 'glove'

            if self.mode == 'glove':
                glove2word2vec(self.trained_vectors, self.temp)
                self.model = KeyedVectors.load_word2vec_format(self.temp,
                                                               binary=self.binary)
                os.remove(self.temp)

            elif self.mode == 'word2vec':
                self.model = KeyedVectors.load_word2vec_format(self.trained_vectors,
                                                               binary=self.binary,
                                                               encoding='latin-1')
            elif self.mode == 'fasttext':
                self.model = FastText.load_fasttext_format(self.trained_vectors,
                                                           encoding='latin-1')

    def vectors(self, word):

        '''GET VECTOR REPRESENTATION

        Takes as input a string (word or sentence) and
        returns the vector representation based on trained vectors.

        '''

        if self.backend == 'spacy':
            return self.model(word).vector

        elif self.backend == 'gensim':
            return self.model[word]

    def similarity(self, word1, word2):

        '''WORD SIMILARITY
        Takes as input two words and returns similarity.

        PARAMS
        ------
        word1 :: a string
        word2 :: another string


        EXAMPLE
        -------
        get = Signs(backend='gensim',
        trained_vectors="e:/signs_dev/vectors/glove/glove.twitter.27B.25d.txt")
        get.similarity('donald', 'trump')

        '''

        if self.backend == 'gensim':
            return self.model.similarity(word1, word2)

        if self.backend == 'spacy':

            a = nlp(word1)
            b = nlp(word2)

            return a.similarity(b)



from signs.vectorize.spacy import spacy_load, spacy_word2vec


def run_test():

    model = spacy_load()
    model = spacy_load('en')

    test = round(sum(spacy_word2vec('test', model)), 2)
    test2 = round(sum(spacy_word2vec('testing sentence', model)), 2)

    if test != 2.41:
        raise ValueError('spacy_word2vec() test failed with %f' % test)

    if test2 != -0.72:
        raise ValueError('spacy_word2vec() test failed with %f' % test)



import pandas as pd
from .converters import embeds_to_text
from kerasplotlib.text import text


class Preds:

    def __init__(self, x_test, y_test, word_index, model):

        # temp values
        self.model = model
        self.x_test = x_test
        self.y_test = y_test
        self.word_index = word_index

        self.results = self._preds_df()

        # delete temp values
        del self.word_index, self.y_test, self.x_test, self.model

    def _preds_df(self):

        '''
        x_test :: the test data (already embedded)
        y_test :: truth values for the test data
        word_index :: the word index used for creating the embeddings

        '''

        results = pd.DataFrame({
                    'text': embeds_to_text(self.x_test, self.word_index),
                    'pred': [i[0] for i in self.model.predict(self.x_test)],
                    'truth': self.y_test
                            })

        return results

    def summary(self, sensitivity=.1, n=5):

        pos = self.results[self.results.pred > 1 - sensitivity].sample(n, replace=True)
        text(pos, 'text', title='Clear Positive')

        mid = self.results[self.results.pred.between(0.5 - (sensitivity / 2), 0.5 + (sensitivity / 2))].sample(n, replace=True)
        text(mid, 'text', title='Close Call', max_rows=5)

        neg = self.results[self.results.pred < sensitivity].sample(n, replace=True)
        text(neg, 'text', title='Clear Negative', max_rows=5)

    def falses(self, sensitivity=.1, n=5):

        # false positive
        fp = self.results[self.results.truth == 0][self.results.pred > 1 - sensitivity].sample(n, replace=True)
        text(fp, 'text', title='False Positives')

        # false negatives
        fn = self.results[self.results.truth == 1][self.results.pred < sensitivity].sample(n, replace=True)
        text(fn, 'text', title='False Negatives')



import spacy as sp


def spacy_load(trained_vectors='en'):
    
    '''LOAD SPACY MODEL 
    
    The default option 'en' loads the spacy default vectors.
    Alternative a local vector file can be loaded or one of the 
    other spacy vectors.
    
    '''
    
    return sp.load(trained_vectors)


def spacy_word2vec(string, model):
    
    '''GET VECTOR REPRESENTATION
    
    Takes as input a string (word or sentence) and 
    returns the vector representation based on trained vectors.
    
    '''
    
    return model(string).vector


from ..preprocess.prepocess_embeddings import keras_embeddings
from ..preprocess.prepocess_embeddings import keras_tokenize
from ..preprocess.prepocess_embeddings import keras_embedding_matrix

from keras.layers import Embedding


class Embeds:

    def __init__(self, trained_vectors):

        self.t_v = trained_vectors
        self.embeddings_index, self.vector_dims = keras_embeddings(self.t_v)

    def _matrix(self):

        self.x, self.vocab_size, self.word_index = keras_tokenize(self.docs)
        self.embedding_matrix = keras_embedding_matrix(self.word_index,
                                                       self.vector_dims,
                                                       self.embeddings_index)

    def layer(self, docs):

        self.docs = docs

        self._matrix()

        embeddding_layer = Embedding(self.vocab_size,
                                     self.vector_dims,
                                     weights=[self.embedding_matrix],
                                     input_length=self.x.shape[1],
                                     trainable=False)

        return embeddding_layer, self.x



from numpy import array, transpose

from keras.models import Sequential
from keras.layers import Dense, Dropout, Flatten, Embedding
from kerasplotlib import TrainingLog


def mlp(x,
        y,
        vocab_size,
        vector_dims,
        embedding_matrix,
        epochs=50,
        layers=0,
        dropout=0,
        batch_size=10,
        inner_neurons=None,
        loss='binary_crossentropy',
        x_test=None,
        y_test=None):

    model = Sequential()
    model.add(Embedding(vocab_size,
              vector_dims,
              weights=[embedding_matrix],
              input_length=x.shape[1],
              trainable=False))
    model.add(Flatten())
    model.add(Dropout(dropout))

    # add layers
    model = layers_dense(model, layers, dropout, inner_neurons)

    model.add(Dense(1, activation='sigmoid'))
    model.compile(optimizer='adam', loss=loss, metrics=['acc'])
    model.fit(x, y,
              epochs=epochs,
              batch_size=batch_size,
              verbose=0,
              validation_split=.3,
              callbacks=[TrainingLog()])
    # loss, accuracy = model.evaluate(x, y, verbose=0)

    return model


def layers_dense(model, layers, dropout, inner_neurons):

    if layers > 0:

        for i in range(layers):
            model.add(Dense(inner_neurons))
            model.add(Dropout(dropout))

    return model



def embeds_to_text(x_test, word_index):

    '''EMBEDS TO TEXT CONVERTER

    Takes in keras embeddings and converts it back to text.
    Note that the resulting strings are limited by max_words
    from tokenization. Use embeds_to_original() to get the full
    string instead.'''

    l = []

    for i in range(len(x_test)):

        inv_map = {v: k for k, v in word_index.items()}
        words = list(map(inv_map.get, x_test[i]))
        sentence = ' '.join([x for x in words if x is not None])
        l.append(sentence)

    return l


def embeds_to_original():

    return



#! /usr/bin/env python
#
# Copyright (C) 2018 Mikko Kotila

import os

DESCRIPTION = "Talos Hyperparameter Scanner for Keras"
LONG_DESCRIPTION = """\
Signs is a utility for text preprocessing, vectorizing, and analysis
such as semantic similarity, mainly for the purpose of using unstructured
data in deep learning models.
"""

DISTNAME = 'signs'
MAINTAINER = 'Mikko Kotila'
MAINTAINER_EMAIL = 'mailme@mikkokotila.com'
URL = 'http://autonom.io'
LICENSE = 'MIT'
DOWNLOAD_URL = 'https://github.com/autonomio/signs/'
VERSION = '0.1.2'

try:
    from setuptools import setup
    _has_setuptools = True
except ImportError:
    from distutils.core import setup


def check_dependencies():

    install_requires = []

    try:
        import kerasplotlib
    except ImportError:
        install_requires.append('kerasplotlib')

    try:
        import wrangle
    except ImportError:
        install_requires.append('wrangle')

    try:
        import numpy
    except ImportError:
        install_requires.append('numpy')

    try:
        import pandas
    except ImportError:
        install_requires.append('pandas')

    try:
        import spacy
    except ImportError:
        install_requires.append('spacy')

    try:
        import keras
    except ImportError:
        install_requires.append('keras')

    try:
        import gensim
    except ImportError:
        install_requires.append('gensim')

    try:
        import tensorflow
    except ImportError:
        install_requires.append('tensorflow')
        
    return install_requires


if __name__ == "__main__":

    install_requires = check_dependencies()

    setup(name=DISTNAME,
          author=MAINTAINER,
          author_email=MAINTAINER_EMAIL,
          maintainer=MAINTAINER,
          maintainer_email=MAINTAINER_EMAIL,
          description=DESCRIPTION,
          long_description=LONG_DESCRIPTION,
          license=LICENSE,
          url=URL,
          version=VERSION,
          download_url=DOWNLOAD_URL,
          install_requires=install_requires,
          packages=['signs',
                    'signs.preprocess',
                    'signs.vectorize',
                    'signs.tests',
                    'signs.grams',
                    'signs.utils',
                    'signs.models',
                    'signs.similarity'],

          classifiers=[
                'Intended Audience :: Science/Research',
                'Programming Language :: Python :: 3.6',
                'License :: OSI Approved :: MIT License',
                'Topic :: Scientific/Engineering :: Human Machine Interfaces',
                'Topic :: Scientific/Engineering :: Artificial Intelligence',
                'Topic :: Scientific/Engineering :: Mathematics',
                'Operating System :: POSIX',
                'Operating System :: Unix',
                'Operating System :: MacOS'])

os.system("python -m spacy download en")



import re
import string

class Clean:
    
    def __init__(self, text):
        
        self.text = text
        
    def low(self):
        
        '''make string lowercase'''

        return self.text.lower()

    def caps(self):
        
        '''make string uppercase'''
        
        return self.text.upper()
    
    def punct(self):
        
        '''remove special characters'''
        
        return re.sub('['+string.punctuation+']', '', self.text)
    
    def leadtrail(self):
        
        '''remove trailing and leading whitespace
        
        NOTE: this also removes the last line break'''
        
        return self.text.strip()
    
    def linebreaks(self):
        
        '''remove linebreaks'''

        return self.text.replace('\n', ' ').replace('\r', '')
        
    def emoji(self):
        
        '''remove emojis'''
    
        emoji_pattern = re.compile("["
                               u"\U0001F600-\U0001F64F"  
                               u"\U0001F300-\U0001F5FF"  
                               u"\U0001F680-\U0001F6FF"  
                               u"\U0001F1E0-\U0001F1FF" 
                               u"\U00002702-\U000027B0"
                               u"\U000024C2-\U0001F251"
                               "]+", flags=re.UNICODE)
        return emoji_pattern.sub(r'', self.text)


from ..preprocess.clean import Clean


def run_test():

    p = Clean(' Jack is a green 😂😂😂 cat... \n with a hat \n  ')

    if p.low() != ' jack is a green 😂😂😂 cat... \n with a hat \n  ':
        raise ValueError('low() test failed')

    if p.caps() != ' JACK IS A GREEN 😂😂😂 CAT... \n WITH A HAT \n  ':
        raise ValueError('caps() test failed')

    if p.punct() != ' Jack is a green 😂😂😂 cat \n with a hat \n  ':
        raise ValueError('punct() test failed')

    if p.leadtrail() != 'Jack is a green 😂😂😂 cat... \n with a hat':
        raise ValueError('leadtrail() test failed')

    if p.linebreaks() != ' Jack is a green 😂😂😂 cat...   with a hat    ':
        raise ValueError('linebreaks() test failed')

    if p.emoji() != ' Jack is a green  cat... \n with a hat \n  ':
        raise ValueError('linebreaks() test failed')
